using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Drawing.Printing;

namespace ScreenSnapper
{
    public partial class ImageDisplayForm : Form
    {
        private Image _displayImage;
        private string _filePath;
        private bool _displayCompleteImage;

        public Image DisplayImage
        {
            get
            {
                return _displayImage;
            }
            set
            {
                _displayImage = value;
            }
        }

        public ImageDisplayForm(Point location, Image image, string filePath, bool displayCompleteImage)
        {
            _displayImage = image;
            _filePath = filePath;
            _displayCompleteImage = displayCompleteImage;

            InitializeComponent();

            if (displayCompleteImage)
            {
                GraphicsUnit pixels = GraphicsUnit.Pixel;
                RectangleF imageBounds = _displayImage.GetBounds(ref pixels);
                Size size = new Size((int)imageBounds.Width, (int)imageBounds.Height);
                ClientSize = size;
                imageLabel.ClientSize = size;
                imageLabel.Location = new Point(0, 0);
            }

            Location = location;

            imageLabel.Image = _displayImage;
        }

        private void imageLabel_MouseDown(object sender, MouseEventArgs e)
        {
            if (e.Button == MouseButtons.Left)
            {
                Keys keys = Control.ModifierKeys;
                if (keys != Keys.Shift)
                {
                    DataObject dataObject = new DataObject(DataFormats.FileDrop, new string[] { _filePath });
                    imageLabel.DoDragDrop(dataObject, DragDropEffects.Copy);
                }
                else
                {
                    DataObject dataObject = new DataObject(DataFormats.Bitmap, _displayImage);
                    imageLabel.DoDragDrop(dataObject, DragDropEffects.Copy);
                }
            }
        }

        private void copyToolStripMenuItem_Click(object sender, EventArgs e)
        {
            Clipboard.SetImage(_displayImage);
        }

        private void printToolStripMenuItem_Click(object sender, EventArgs e)
        {
            PrintDocument print = new PrintDocument();
            print.PrintPage += new PrintPageEventHandler(ImageDisplayForm_PrintPage);
            print.Print();
        }

        private void printPreviewToolStripMenuItem_Click(object sender, EventArgs e)
        {
            PrintDocument print = new PrintDocument();
            print.PrintPage += new PrintPageEventHandler(ImageDisplayForm_PrintPage);
            PrintPreviewDialog preview = new PrintPreviewDialog();
            preview.Document = print;
            preview.ShowDialog(this);
        }
        
        private void ImageDisplayForm_PrintPage(Object sender, PrintPageEventArgs ev)
        {
            // We might do something with the image, so clone it first.
            Image image = (Image) _displayImage.Clone();

            int pageWidth = ev.MarginBounds.Width;
            int pageHeight = ev.MarginBounds.Height;

            GraphicsUnit pixels = GraphicsUnit.Pixel;
            RectangleF imageBounds = image.GetBounds(ref pixels);
            int imgWidth = (int) imageBounds.Width;
            int imgHeight = (int) imageBounds.Height;
            double imgRatio = ((double) imgWidth) / ((double) imgHeight);
            // Wider than high?
            if (imgRatio > 1.0)
            {
                // Rotate image 90
                image.RotateFlip(RotateFlipType.Rotate90FlipNone);
                int temp = imgWidth;
                imgWidth = imgHeight;
                imgHeight = temp;
            }

            // Now position the image in the center of the page
            int imgPageWidth = pageWidth;
            int imgPageHeight = (int) (((double) pageWidth) / ((double) imgWidth) * ((double) imgHeight));
            if (imgPageHeight > pageHeight)
            {
                imgPageHeight = pageHeight;
                imgPageWidth = (int)(((double)pageHeight) / ((double)imgHeight) * ((double)imgWidth));
            }

            int leftOffset = (pageWidth - imgPageWidth) / 2;
            int topOffset = (pageHeight - imgPageHeight) / 2;

            ev.Graphics.DrawImage(image, ev.MarginBounds.Left + leftOffset, ev.MarginBounds.Top + topOffset, imgPageWidth, imgPageHeight);
        }

        private void saveAsToolStripMenuItem_Click(object sender, EventArgs e)
        {
            string extension = ".png";
            string filter = "PNG Files (*.png)|*.png";
            if (_filePath.EndsWith(".jpg"))
            {
                extension = ".jpg";
                filter = "JPG Files (*.jpg)|*.jpg";
            }
            saveFileDialog.AddExtension = true;
            saveFileDialog.DefaultExt = extension;
            saveFileDialog.Filter = filter;
            if (DialogResult.OK == saveFileDialog.ShowDialog(this))
            {
                System.IO.File.Copy(_filePath, saveFileDialog.FileName, true);
            }
        }

        private void copyFullPathToolStripMenuItem_Click(object sender, EventArgs e)
        {
            Clipboard.SetText(_filePath);
        }
   }
}